Require Import ssreflect ssrbool eqtype ssrnat ssralg ssrnum complex.


(******************************************************************************)
(*                                                                            *)
(* This file is a complement of complex.v of SSReflect library. In particular *)
(* he contains lemmas to facilitate the correspondance between a real number  *)
(* and a complex number with a null imaginary part.                           *)
(******************************************************************************)

 
Import GRing.Theory.
Import Num.Theory.
Import Num.Def.

Section prelemme.

Variable R : rcfType.
Local Notation C := (complex R).

Open Scope ring_scope.

Lemma Im0 (a : C) : a \is Num.real -> Im a = 0.
Proof.
rewrite realE=> /orP []; first exact: ger0_Im.
by rewrite lecE=> /andP [] /eqP <-.
Qed.

Lemma Re_ge0 (a : C) : a \is Num.real -> (0 <= Re a) = (0 <= a).
Proof. by move/Im0=> Ha; rewrite lecE Ha eqxx. Qed.

Lemma Re_gt0 (a : C) : a \is Num.real -> (0 < Re a) = (0 < a).
Proof. by move/Im0=> Ha; rewrite ltcE Ha eqxx. Qed.

Lemma real_Re_eq0 (x : C) : x \is Num.real -> (Re x == 0) = (x == 0).
Proof. by rewrite eq_complex=> /Im0 ->; rewrite eqxx andbT. Qed.

Lemma real_lecE (x y : C) : x \is Num.real -> y \is Num.real ->
   (x <= y) = (Re x <= Re y).
Proof. by move/Im0=> xIm0 / Im0 yIm0; rewrite lecE xIm0 yIm0 eqxx. Qed.

Lemma real_ltcE (x y : C) : x \is Num.real -> y \is Num.real ->
   (x < y) = (Re x < Re y).
Proof. by move/Im0=> xIm0 / Im0 yIm0; rewrite ltcE xIm0 yIm0 eqxx. Qed.
 
Lemma ReK (a : C) : a \is Num.real -> (Re a)%:C = a.
Proof. by move=>/Im0 Ha; apply/eqP; rewrite eq_complex Ha !eqxx. Qed.

Lemma ReM (a b : C) : Re (a * b) = Re a * Re b - Im a * Im b.
Proof. by case: a b=> Ra Ia [] Rb Ib. Qed.

Lemma real_ReM (a b : C) : a\is Num.real -> b \is Num.real ->
  Re ( a * b) = Re a * Re b.
Proof. by rewrite ReM => /Im0 -> /Im0 ->; rewrite mulr0 subr0. Qed.

Lemma real_ReX (a : C) n : a\is Num.real -> Re (a ^+ n) = (Re a) ^+ n.
Proof.
by move=> Ha; elim: n=> // n Ihn; rewrite !exprS -Ihn real_ReM ?realX.
Qed.

Lemma real_ReV (a : C) : a \is Num.real -> Re (a^-1) = (Re a)^-1.
Proof.
case: (altP (a=P 0))=> [->|anz ar]; first by rewrite !invr0.
have{anz}:= anz; have/Im0 := ar; have/real_Re_eq0 {ar} <- := ar.
case: a => Ra Ia /= -> ar. 
by rewrite expr0n addr0 expr2 invrM ?mulVKr // unitfE.
Qed.

Lemma normc_real (x : R) : `|x%:C| = `|x|%:C.
Proof.
by rewrite normcE sqrtc_sqrtr ?mulcJ_ge0 //= mul0r subr0 -expr2 sqrtr_sqr.
Qed.

Lemma Re_nat (n : nat) : Re (n%:R : C) = (n%:R : R).
Proof. by elim: n=> // n Ihn; rewrite -addn1 !natrD raddfD /= Ihn. Qed.

Lemma natC (n : nat) : ((n%:R : R)%:C)=(n%:R : C).
Proof. by apply/eqP; rewrite eq_complex /= Re_nat Im0 ?eqxx // realn. Qed.

End prelemme.
